%CURVATURE   Curvature in grey value images
%            in nD without segmentation
%
% SYNOPSIS:
%  out = curvature(in, dimen, sg, st)
%
%  dimen: 'line' assuming a line-like structure
%                being 1D, one curvaure is returned
%         'surface' assuming a plan-like structure
%                being (n-1)D, (n-1) principle curvatures are returned
%  sg: sigma of the gradient derivative
%  st: sigma of the tensor smoothing
%
% DEFAULTS:
% dimen = 'line'
%  sg = 1   
%  st = 4
%
% NOTE: For 4D and higher the output will be subsampled by st
%
% SEE ALSO:
%  isophote_curvature, orientation4d
%
% LITERATURE: Lines
%  M. van Ginkel, J. van de Weijer, P.W. Verbeek, and L.J. van
%   Vliet.  Curvature estimation from orientation fields.  In B.K.
%   Ersboll and P. Johansen, editors, SCIA'99, Proc. 11th Scandi-
%   navian Conference on Image Analysis (Kangerlussuaq, Green-
%   land), pages 545-551. Pattern Recognition Society of Den-
%   mark, Lyngby, June 7-11 1999.
%
%  B.Rieger, L.J. van Vliet, Curvature of n-dimensional space
%   curves in grey-value images, IEEE Transactions on Image 
%   Processing, 2002
%
% LITERATURE: Planes
%  B. Rieger, F.J. Timmermans, van L.J. Vliet and P.W. Verbeek,
%  Curvature estimation of surfaces in 3D grey-value images,
%  ICPR'02, Proc. 16th Int. Conf. on Pattern Recognition, 2002 
 
% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger, March 2002.
% Nov, 2002, added nD code
% Dec, 2002, added surface curvatures

function out = curvature(varargin)
d = struct('menu','Analysis',...
  'display','Curvature',...
  'inparams',struct('name',       {'in',   'opt','sg','st'},...
  'description',{'Input image','','Sigma of Derivative', 'Sigma of Tensor'},...
   'type',       {'image','option','array','array'},...
   'dim_check',  {0,0,1,1},...
   'range_check',{[],{'line','surface'},'R+','R+'},...
   'required',   {1,0,0,0},...
   'default',    {'a','line',1,4}...
  ),...
  'outparams',struct('name',{'out'},...
   'description',{'Output image'},...
   'type',{'image'}...
   )...
 );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in,opt,sg,st] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

N = ndims(in);

switch opt
   case 'line'
      switch N
         case 2
            %use MvG code
            phi=structuretensor(in,sg,st,{'orientation'});
            phimap=exp(2j*phi);
            phicnj=conj(phimap);
            phidx=0.5*imag(phicnj*(dx(real(phimap),1)+j*dx(imag(phimap),1)));
            phidy=0.5*imag(phicnj*(dy(real(phimap),1)+j*dy(imag(phimap),1)));
            out=-sin(phi)*phidx+cos(phi)*phidy;
         case 3
            %use BR code
            %use analytic solution for GST and 5D Knutsson mapping
            [phi,theta]=structuretensor3d(in,sg,st,{'phi3','theta3'});
            Knut_T = newimar(5);
            Knut_T{1} = (sin(theta))^2*cos(2*phi);
            Knut_T{2} = (sin(theta))^2*sin(2*phi);
            Knut_T{3} = sin(2*theta)*cos(phi);
            Knut_T{4} = sin(2*theta)*sin(phi);
            Knut_T{5} = sqrt(3)*((cos(theta))^2-1/3);
            DM1 = newimar(5,3);
            sk = 1;
            for ii= 1:5
               DM1{ii,1}=dx(Knut_T{ii},sk);
               DM1{ii,2}=dy(Knut_T{ii},sk);
               DM1{ii,3}=dz(Knut_T{ii},sk);
            end
            clear Knut_T
            T = newimar(3);
            T{1} = sin(phi)*sin(theta);
            T{2} = cos(phi)*sin(theta);
            T{3} = cos(theta);
            out = 0.5 *norm(DM1 * T);
         otherwise
            if N==4 %faster implementation
               T = orientation4d(in,sg,st);
            else
               G = gst_subsample(in,sg,st);
               [ew,TT]=jacobi(G);
               clear G ew
               T=TT{:,N};%smallest eigenvalue/vector
               clear TT
             end
             M = T*T';
             DM = newimar(N*N,N);
             sigma   = ones(N,1);
             process = ones(N,1);
             order  = zeros(N,1);
             for ii= 1:N*N
             for jj =1:N
               order(jj) = 1;
               DM{ii,jj}=dip_gauss(M{ii},process,sigma,order);
               order(jj) = 0;
             end
             end
             clear M 
             out = norm(DM*T)./sqrt(2);
         end
   case 'surface'
      if N < 3
         error('Input image should be at least 3D.');
      end
            
      out = newimar(N-1);
      switch N
         case 3
            sk=1;
            %3D code, fast eig analysis + 5d Knutsson mapping
            g = gradient(in ,sg);
            G = smooth(g*g',st);
            [v1 v2 v3]=dip_symmetriceigensystem3(G{1,1},G{1,2},G{1,3},G{2,2},...
               G{2,3},G{3,3},{'v1','v2','v3'}); 
            T = newimar(5);
            T{1} = v1{1}^2-v1{2}^2;
            T{2} = 2*v1{1}*v1{2};
            T{3} = 2*v1{1}*v1{3};
            T{4} = 2*v1{2}*v1{3};
            T{5} =  1/sqrt(3)*(2*v1{3}^2-v1{1}^2-v1{2}^2);
            DM = newimar(5,3);
            for jj= 1:5 
               DM{jj,1}=dx(T{jj},sk); 
               DM{jj,2}=dy(T{jj},sk); 
               DM{jj,3}=dz(T{jj},sk); 
            end 
            clear T
            Knut_T1=DM*v2;
            out{1} = norm(Knut_T1)/2;
            Knut_T1=DM*v3;
            out{2} = norm(Knut_T1)/2;
         otherwise
            %nD code
            G = gst_subsample(in,st,sg);
            [ew,l] = jacobi(G);
            clear G ew
            M=l{:,1}*l{:,1}';%surface normal
            DM = newimar(N*N,N);
            sigma   = ones(N,1);
            process = ones(N,1);
            order  = zeros(N,1);
            for ii= 1:N*N
            for jj =1:N
               order(jj) = 1;
               DM{ii,jj}=dip_gauss(M{ii},process,sigma,order);
               order(jj) = 0;
            end
            end
            clear M
      
            for ii=1:N-1
               out{ii} = norm(DM*l{:,ii+1})./sqrt(2);
            end 
         end
      otherwise
         error('Unkown option.');
         
end


