%GAUSSIANBLOB  Adds a Gauss shaped spot to an  image
%
% SYNOPSIS:
%  image_out = gaussianblob(image_in, coordinates, sigma, strength, domain)
%
%  image_in:    input image, where the spot will be set
%  image_out    = image_in + spot
%  domain:      spatial or frequency
%    SPATIAL
%      coordinates: array containing the location of the spot (0,image size)
%      sigma:       the sigma of the isotropic gauss
%      strength:    the integrated intensity of the spot
%      spot = strength * (2pi sigma^2)^(-imagedimension/2) * exp(-1/2 (x/sigma)^2)
%    FREQ
%      coordinates: array containing the location of the spot (-0.5,0.5)
%      sigma:       the sigma of the isotropic gauss (in frequency domain)
%      strength:    the integrated intensity of the spot
%      spot = 1/(sqrt(sum(pixels))) * exp(-1/2 (sigma 2 pi f)^2)
%
% DEFAULTS:
%  sigma = 2
%  strength = 255
%  domain = 'spatial'

% (C) Copyright 1999-2001               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger, Apr 2001

function out = gaussianblob(varargin)

d = struct('menu','Generation',...
           'display','Add Gaussian blob',...
           'inparams',struct('name',       {'in',         'coord',    'sigma',      'strength' 'domain'},...
                             'description',{'Input image','Coordinates','Sigma','Strength', 'Domain'},...
                             'type',       {'image',      'array',    'array',     'array', 'option'},...
                             'dim_check',  {0,            1,           0,           0, 0},...
                             'range_check',{[],           'R',        'R+',         'R', {'spatial','freq'}},...
                             'required',   {1,            1,           0,           0, 0},...
                             'default',    {'a',          '[128 128]',   2,           255, 'spatial'}...
                            ),...
           'outparams',struct('name',{'out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
         );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in,coord,sigma,strength,domain] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end


di = ndims(in);
switch domain
   case 'spatial'
      if any(coord > size(in)) | any(coord < 0)
         error('Specified coordinates out of image.');
      end
      origin = 'corner';
      normalisation = (sigma*sqrt(2*pi))^di;
      sigma = 1/sigma;
   case 'freq'
      coord=2*pi*coord;
      if any(abs(coord) > pi)
         error('Specified coordinates out of image.');
      end
      origin = 'radfreq';
      % make sure ift(frequency)=spatial therefore normalisation 1/sqrt(N)
      normalisation = sqrt(prod(size(in)));
   otherwise
      error('Unkown domain.');
end

exponent = (createramp(size(in),1,origin)-coord(1))^2;
for ii = 2:di
   exponent = exponent + (createramp(size(in),ii,origin)-coord(ii))^2;
end
out = in + strength/normalisation * exp(-0.5*sigma^2*exponent);
