%WRITEIM   Write grey-value or color image to file
%
% SYNOPSIS:
%  writeim(image_in,filename,format,compression)
%
% PARAMETERS:
%  image_in:    image to write to file.
%  filename:    string with name of file, optionally with path and extension.
%  format:      any one string of: 'ICSv1', 'ICSv2', 'TIFF', 'PS', 'EPS', 'CSV',
%               'GIF' or 'FLD'.
%  compression: 1 or 'yes' to use comression or 0 or 'no' not to. Not all
%               formats support compression. The compression method used is
%               the default for that file type. It is not possible to change
%               the compression method from within this function.
%
% DEFAULTS:
%  format = 'ICSv2'
%  compression = 'yes'
%
% NOTES:
%  To get a current list of file formats, use the function DIPIO_GETIMAGEWRITEFORMATS,
%  which also gives a short description of each format.
%
%  Format 'TIF' is an alias for 'TIFF'. Likewise, 'ICS' is an alias for 'ICSv2'.

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, June 2000.
% January 2002:  Changed compression default to 1 (BR)
% February 2002: Removed static list of file formats.
% July 2002:     Allowing color images as input. This function now substitutes READCOLORIM.
%                Using new DIPIO_IMAGEWRITECOLOUR function.
%                ICS version 2 is now the default (i.e. the 1-file format).

function out = writeim(varargin)

frmts = dipio_getimagewriteformats;

% The large number of spaces in the 'description' field for 'format' causes the display of
% the 'frmts' list to be wider. The data in the list is not used to determine the width of
% the controls, only the description is.
d = struct('menu','File I/O',...
   'display','Write image',...
   'inparams',struct('name',{'image_in','filename','format','compression'},...
      'description',{'Image to write','Name of the file to write','File format','Use compression?'},...
      'type',       {'image','outfile','option','boolean'},...
      'dim_check',  {0,      0,         0,       0},...
      'range_check',{[],     '*.*',     frmts,   []},...
      'required',   {1,      1,         0,       0},...
      'default',    {'a',    '',        'icsv2', 1}...
      )...
   );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
%%% aliases for elements in the 'format' list.
if nargin>=3 & ischar(varargin{3})
   if strcmpi(varargin{3},'ics')
      varargin{3} = 'icsv2';
   elseif strcmpi(varargin{3},'ics1')
      varargin{3} = 'icsv1';
   elseif strcmpi(varargin{3},'ics2')
      varargin{3} = 'icsv2';
   elseif strcmpi(varargin{3},'tif')
      varargin{3} = 'tiff';
   end
end
%%%
try
   [image_in,filename,format,compression] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

if isa(image_in,'dip_image_array')
   if iscolor(image_in)
      switch colorspace(image_in)
         case 'grey'
            photometric = 'grayvalue';
         case 'RGB'
            photometric = 'RGB';
         case 'L*a*b*'
            photometric = 'CIElab';
         case 'CMYK'
            photometric = 'CMYK';
         case 'YCbCr'
            photometric = 'YCbCr';
         otherwise
            image_in = colorspace(image_in,'RGB');
            photometric = 'RGB';
      end
      image_in = cat(3,image_in);
      try
         dipio_imagewritecolour(image_in,filename,photometric,format,compression);
      catch
         error(firsterr)
      end
   else
      error('Input is an array of images.')
   end
else
   try
      dipio_imagewrite(image_in,filename,format,compression);
   catch
      error(firsterr)
   end
end

